///////////////////////////////////////////////////////////////////////////
//                         Multimedia System 1st Beta                     //
//                              for TMT Pascal                            //
//                      (MS-DOS 32-bit protected mode)                    //
//                             by Vadim Bodrov                            //
//                      Based on Sound System Release 4                   //
//                                 by                                     //
//                     ͻ   Frontman    Ŀ    Ŀ                 //
//                                of                                  //
//                          ͻͻ                     //
//                             ͼ                              //
//                     ͼ    ͼ                      //
//                                                                        //
//           Interface unit for TMT Pascal (http://www.tmt.com)           //
////////////////////////////////////////////////////////////////////////////
unit MMedia;

{$w-,r-,q-,i-,t-,x+,v-,a+,oa-,opt+}
interface

const
  Sb       = 0;
  SbPro    = 1;
  Sb16     = 2;
  Pas      = 3;
  Gus      = 4;
  Nos      = 255;

var
  SbDrv:   String := 'sbmod.drv';
  Sb16Drv: String := 'sb16mod.drv';
  PasDrv:  String := 'pasmod.drv';
  GusDrv:  String := 'usmod.drv';
  NosDrv:  String := 'nosnd.drv';

var
  mmError: DWord;
  //  0      - Ok
  //  1      - File not found
  //  2      - IO error
  //  3      - Not enaugh DOS memory
  //  4      - Invalid parameter
  //  5      - Not a driver

function  GetDriverID(var ID: Byte): Boolean;
procedure InstallSoundDriver(DrvId: Byte);
procedure DeInstallSoundDriver;
function  InitConfig: Boolean;
function  LoadModule(Name: String): Boolean;
function  LoadSample(Name: String): Word;
function  PlayModule: Boolean;
function  PlaySample(Handle: Word; Freq: Word): Boolean;
function  StopModule: Boolean;
function  UnloadModule: Boolean;
function  StopSample: Boolean;
function  SetSongLoop(PatternOrder: Byte): Boolean;

implementation

uses DPMI;

var
  SndSeg:   DWord:=0;
  SndAddr:  DWord:=0;
  CallOffs: Word:=0;

var
  Regs: TRmRegs;

function GetDriverID(var ID: Byte): Boolean;
var
  f: File;
  Drv: array [0..13] of Byte;
begin
{$I-}
  Assign(f, 'mmdrv.cfg');
  Reset(f);
  BlockRead(f, Drv, 14);
{$I+}
  if IOResult = 0 then begin
    Result := true;
    ID := Drv[6];
  end else
    Result := false;
{$I-}
  Close(f);
end;

procedure InstallSoundDriver(DrvId: Byte);
var
  Name: String;
  f:    File;
begin
  case DrvId of
    Sb,SbPro: Name := 'sbmod.drv';
    Sb16    : Name := 'sb16mod.drv';
    Pas     : Name := 'pasmod.drv';
    Gus     : Name := 'usmod.drv';
    Nos     : Name := 'nosnd.drv';
  end;
  SndSeg := DOSMemoryAlloc(32000);
  SndAddr := DWord(SndSeg) shl 4;
  if SndSeg = 0 then
  begin
    mmError:=3;
    exit;
  end;
  Assign(f, Name);
{$I-}
  Reset(f);
{$I+}
  if IOResult <> 0 then begin
   mmError := 1;
   exit;
  end;
{$I-}
  BlockRead(f, Pointer(SndAddr + $100)^, FileSize(f));
{$I+}
  if IOResult <> 0 then begin
    mmError := 2;
    DOSMemoryFree(SndSeg);
    exit;
  end;
{$I-}
  Close(f);
  CallOffs := Word(Pointer(SndAddr + $100)^);
  mmError := 0;
 end;

procedure DeInstallSoundDriver;
begin
  if SndSeg > 0 then
   begin
     DOSMemoryFree(SndSeg);
     SndSeg := 0;
  end;
end;

procedure FreeDriver(Segment: Word);
begin
  if DOSMemoryFree(Segment) then
    mmError := 0
  else
    mmError := 4;
end;

function InitConfig: Boolean;
begin
  ClearRmRegs(Regs);
  Regs.CS := SndSeg;
  Regs.IP := $100 + CallOffs;
  Regs.CX := 1;
  CallRealModeFar(Regs);
  Result := Boolean(Regs.Flags xor 1);
end;

function LoadModule(Name: String): Boolean;
var
  i: DWord;
  zName: array [0..255] of char;
begin
  FillChar(zName[0], 0, SizeOf(zName));
  for i := 1 to Length(Name) do
    zName[i-1] := Name[i];
  zName[i] := char(0);
  Move(zName[0], Pointer(SndAddr)^, SizeOf(zName));
  ClearRmRegs(Regs);
  Regs.CS := SndSeg;
  Regs.IP := $104 + CallOffs;
  Regs.DS := SndSeg;
  Regs.DX := 0;
  CallRealModeFar(Regs);
  Result := Boolean(Regs.Flags xor 1);
end;

function LoadSample(Name: String): Word;
var
  i: DWord;
  zName: array [0..255] of char;
begin
  FillChar(zName[0], 0, SizeOf(zName));
  for i := 1 to Length(Name) do
    zName[i-1] := Name[i];
  zName[i] := char(0);
  Move(zName[0], Pointer(SndAddr)^, SizeOf(zName));
  ClearRmRegs(Regs);
  Regs.CS := SndSeg;
  Regs.IP := $114 + CallOffs;
  Regs.DS := SndSeg;
  Regs.DX := 0;
  Regs.CX := $80;
  CallRealModeFar(Regs);
  Result := Regs.AX;
end;

function PlayModule: Boolean;
begin
  ClearRmRegs(Regs);
  Regs.CS := SndSeg;
  Regs.IP := $108 + CallOffs;
  CallRealModeFar(Regs);
  Result := Boolean(Regs.Flags xor 1);
end;

function PlaySample(Handle: Word; Freq: Word): Boolean;
begin
  ClearRmRegs(Regs);
  Regs.CS := SndSeg;
  Regs.IP := $118 + CallOffs;
  Regs.CX := Freq;
  Regs.BX := Handle;
  Regs.AL := 127;
  CallRealModeFar(Regs);
  Result := Boolean(Regs.Flags xor 1);;
end;

function StopModule: Boolean;
begin
  ClearRmRegs(Regs);
  Regs.CS := SndSeg;
  Regs.IP := $10C + CallOffs;
  CallRealModeFar(Regs);
  Result := Boolean(Regs.Flags xor 1);
end;

function UnloadModule: Boolean;
begin
  ClearRmRegs(Regs);
  Regs.CS := SndSeg;
  Regs.IP := $110 + CallOffs;
  CallRealModeFar(Regs);
  Result := Boolean(Regs.Flags xor 1);
end;

function StopSample: Boolean;
begin
  ClearRmRegs(Regs);
  Regs.CS := SndSeg;
  Regs.IP := $11C + CallOffs;
  CallRealModeFar(Regs);
  Result := Boolean(Regs.Flags xor 1);
end;

function SetSongLoop(PatternOrder: Byte): Boolean;
begin
  ClearRmRegs(Regs);
  Regs.CS := SndSeg;
  Regs.IP := $12C + CallOffs;
  Regs.AL := PatternOrder;
  CallRealModeFar(Regs);
  Result := Boolean(Regs.Flags xor 1);
end;

end.